<?php
	
	// En-tête
	$title = "Les fonctions et procédures en PHP";
	include("../header.php");
	 
?>

<h2>Théorie</h2>

<div class="phrase">
	En programmation, <b>fonctions</b> et <b>procédures</b> permettent de créer 
	des algorithmes réutilisables. Elles permettent de centraliser un ensemble 
	de traitements récurrents afin de ne pas avoir à les coder plusieurs fois. 
	<br/><br/>
	Vous l'aurez compris, PHP nous fournit déjà tout un arsenal de fonctions et 
	procédures prédéfinies. Mais vous pouvez aussi créer vos propres fonctions 
	et procédures. De la sorte, vous pourrez vous épargner parfois de multiples 
	copier/coller inutiles ! Quand vous constatez que vous avez besoin à plusieurs 
	endroits d'effectuer la même série de traitements, c'est très probablement 
	qu'il serait judicieux de créer une fonction ou procédure.
	<br/><br/>
	Une fonction ou une procédure a un <b>nom</b> et on peut l'utiliser à partir 
	du moment où elle a été définie !
	<br/><br/>
	En PHP, pour définir une fonction ou une procédure, on utilise le mot-clef 
	<b>function</b>. 
	<br/><br/>
	La différence entre une procédure et une fonction réside dans le fait que :
	<br/>
	- une procédure ne retourne aucun résultat ;
	<br/>
	- une fonction retourne un et un seul résultat qu'on peut par exemple 
	placer dans une variable. Au sein d'une fonction, on utilise le mot-clef 
	<b>return</b> pour retourner le résultat de la fonction.
	<br/><br/>
	Lorsqu'on utilise* une fonction ou une procédure, on peut lui passer des 
	données. Ce sont ses "variables". On les appelle des <b>paramètres</b>.
	<br/><br/>
	<i>* On dit plutôt qu'on <b>appelle</b> une fonction ou une procédure.</i>
</div>

<h2>Exemples</h2>

<h3>Exemple 1</h3>

<div class="code">
	<code>
		// Création d'une procédure nommée "helloWorld"
		function helloWorld(){
			echo "Bonjour tout le monde !";
		}
		// La procédure "helloWorld" étant définie, on peut l'appeler
		helloWorld();		
	</code>
</div>

<h3>Exemple 2</h3>

<div class="code">
	<code>
		// Création d'une procédure "helloToi" avec 2 paramètres
		function helloToi($prenom, $nom){
			echo "Bonjour " . $prenom . " " . $nom . " !";
		}
		// La procédure "helloToi" étant définie, on peut l'appeler
		helloToi("Will", "SMITH");		
	</code>
</div>

<div class="resultat">
	<?php
		// Création d'une procédure "helloToi" avec 2 paramètres
		function helloToi($prenom, $nom){
			echo "Bonjour " . $prenom . " " . $nom . " !";
		}
		// La procédure "helloToi" étant définie, on peut l'appeler
		helloToi("Jimmy", "PAQUEREAU");
	?>
</div>

<h3>Exemple 3</h3>

<div class="code">
	<code>
		// Création d'une fonction retournant la somme de 2 nombres
		function somme($a, $b){
			return $a + $b;
		}
		// Création puis affichage de deux nombres :
		$nombre1 = 12;
		$nombre2 = 13;
		echo "Nombre 1 => " . $nombre1 . "&lt;br/>";
		echo "Nombre 2 => " . $nombre2 . "&lt;br/>";
		// Récupération puis affichage de la somme des deux nombres
		$somme1et2 = somme($nombre1, $nombre2);
		echo "Somme => " . $somme1et2 . "&lt;br/>";
		// Ou plus rapide...
		echo "Somme => " . somme($nombre1, $nombre2);
	</code>
</div>

<div class="resultat">
	<?php
		// Création d'une fonction retournant la somme de 2 nombres
		function somme($a, $b){
			return $a + $b;
		}
		// Création puis affichage de deux nombres :
		$nombre1 = 12;
		$nombre2 = 13;
		echo "Nombre 1 => " . $nombre1 . "<br/>";
		echo "Nombre 2 => " . $nombre2 . "<br/>";
		// Récupération puis affichage de la somme des deux nombres
		$somme1et2 = somme($nombre1, $nombre2);
		echo "Somme => " . $somme1et2 . "<br/>";
		// Ou plus rapide...
		echo "Somme => " . somme($nombre1, $nombre2);
	?>
</div>

<h3>Exemple 4</h3>

<div class="code">
	<code>
		&lt;?php
			// Création d'une fonction qui, en fonction de l'âge d'une 
			// personne, précise si celle-ci est majeure ou mineure.
			function estMajeur($age){
				if($age >= 18){
					return true;
				}else{
					return false;
				}
			}
		?>
		&lt;form method="POST" action="php/exemples/fonctions.php">
			&lt;label>
				Quel est ton âge ? 
				&lt;?php if(isset($_POST["age"])){ ?>
				&lt;input name="age" type="number" min="0" value="&lt;?=$_POST["age"]?>" /> 
				&lt;?php }else{ ?>
				&lt;input name="age" type="number" min="0" value="0" /> 
				&lt;?php } ?>
			&lt;/label>
			&lt;button>Tester&lt;/button>
		&lt;/form>
		&lt;br/>
		&lt;?php
			// Si l'utilisateur a saisi son âge, on affichage s'il 
			// est majeur ou mineur.
			if(isset($_POST["age"])){
				if(estMajeur($_POST["age"])){
					echo "Tu est majeur.";
				}else{
					echo "Tu est mineur.";
				}
			}
		?>
	</code>
</div>

<div class="resultat">
	<?php
		// Création d'une fonction qui, en fonction de l'âge d'une 
		// personne, précise si celle-ci est majeure ou mineure.
		function estMajeur($age){
			if($age >= 18){
				return true;
			}else{
				return false;
			}
		}
	?>
	<form method="POST" action="php/exemples/fonctions.php">
		<label>
			Quel est ton âge ? 
			<?php if(isset($_POST["age"])){ ?>
			<input name="age" type="number" min="0" value="<?=$_POST["age"]?>" /> 
			<?php }else{ ?>
			<input name="age" type="number" min="0" value="0" /> 
			<?php } ?>
		</label>
		<button>Tester</button>
	</form>
	<br/>
	<?php
		// Si l'utilisateur a saisi son âge, on affichage s'il 
		// est majeur ou mineur.
		if(isset($_POST["age"])){
			if(estMajeur($_POST["age"])){
				echo "Tu est majeur.";
			}else{
				echo "Tu est mineur.";
			}
		}
	?>
</div>

<?php
	
	// Pied de page
	include("../footer.php");
	
?>
