<?php
	
	// En-tête
	$title = "La manipulation d'une base de données en PHP";
	include("../header.php");
	 
?>

<h2>Les bases de données</h2>

<div class="phrase">
	Une <b>base de données</b>, plus exactement une base de données relationnelle, 
	est grossièrement un ensemble de données structurées que l'on peut manipuler. 
	Par manipuler, on entend : ajouter des données, modifier des données, supprimer 
	des données, trouver des données, etc.
	<br/><br/>
	Le <b>SGBD</b> (Système de Gestion de Base de Données) est le logiciel qui conserve 
	et permet de manipuler les bases de données.
	<br/><br/>
	Le serveur web que nous utilisons est WampServer. Or, WampServer intègre un SGBD : 
	<b>MySQL</b>. Si bien que c'est lui que nous utiliserons. WampServer intègre aussi 
	un logiciel facilitant l'utilisation de MySQL : <b>PhpMyAdmin</b>. PhpMyAdmin est 
	tout bonnement une application web. Vous pouvez y accéder en cliquant sur le lien 
	suivant :
	<br/><br/>
	<a class="lien" href="/phpmyadmin">http://localhost/phpmyadmin/</a>
	<br/><br/>
	Grossièrement, une base de données est constituée de "tableaux" qu'on appelle 
	des <b>tables</b>. Une table a des colonnes qu'on appelle classiquement des 
	<b>champs</b>. Chaque champ a un type de données. Un champ au moins doit permettre 
	d'identifier chaque ligne car, en effet, une table comporte également des lignes. 
	Ce champ identifiant est qualifié de <b>clef primaire</b>. On se cantonnera 
	à cela pour le moment...
	<br/><br/>
	Pour plus d'informations concernant les bases de données, on pourra se référer 
	au :
	<br/><br/>
	<a class="lien" href="https://www.orleans-informatique.com/info/stmg/cours/Cours - Q4.pdf" target="_blank"> 
		Cours - Q4 - bases de données et SQL
	</a>
	<br/><br/>
	À présent, créons une base de données appelée "exercices" :
	<br/>
	- accéder à PhpMyAdmin. L'identifiant et le mot de passe par défaut sont 
	respectivement "root" et "" (rien...) ;
	<br/>
	- cliquer sur "Nouvelle base de données" dans le menu de gauche ;
	<br/>
	- saisir "exercices" comme nom de base de données puis cliquer sur le bouton 
	"Créer". La base de données "exercices" apparaît désormais dans le menu à gauche. 
	Actualiser la page si besoin.
	<br/><br/>
	Maintenant que nous avons une base de données, armons-nous d'une unique table ! 
	Vous verrez qu'une base de données constituée d'une seule table, cela ne fait 
	vraiment pas beaucoup. Mais qu'importe...
	<br/><br/>
	Tout d'abord, cliquez sur la base de données "exercices" dans le menu à gauche. 
	Puis, pour créer notre table, suivez les étapes suivantes (attention à bien 
	compléter les formulaires) :
	<br/><br/>
	<img class="capture" src="img/capture-table-creation.jpg" />
</div>

<h2>Les requêtes SQL</h2>

<div class="phrase">
	Pour manipuler les données d'une base de données relationnelle, on utilise 
	des <b>requêtes SQL</b>. Quant au SQL, c'est un langage conçu spécialement 
	pour manipuler des bases de données relationnelles.
	<br/><br/>
	Voici quelques requêtes que vous pouvez tester dans l'onglet "SQL". On peut 
	visualiser les lignes de la table dans l'onglet "Afficher".
	<br/><br/>
	<u>Exemple de requêtes pour insérer des données</u> :
	<br/><br/>
	<code>
		# Insertion d'une question
		INSERT INTO question(question, reponse) 
		VALUES('Quel est la capitale de la France ?', 'paris'); 
		
		# Insertion d'une question
		INSERT INTO question(question, reponse)
		VALUES('En quelle année a été promulgué le code civil ?', '1804'); 
	
		# Insertion d'une question
		INSERT INTO question(question, reponse) 
		VALUES('Quel est le nom de l\'auteur des Rougon-Macquart ?', 'ZoLa'); 
	</code>
	<br/><br/>
	<u>Exemple de requêtes pour supprimer des données</u> :
	<br/><br/>
	<code>
		# Suppression de toutes les questions 
		DELETE FROM question;
		
		# Suppression d'une question en fonction de son numero
		# N.B.: au besoin, changer le n° de ligne
		DELETE FROM question WHERE numero = 3;	
	</code>
	<br/><br/>
	<u>Exemple de requête pour modifier des données</u> :
	<br/><br/>
	<code>
		UPDATE question 
		SET reponse = "zola" 
		WHERE question LIKE 'Quel est le nom de l\'auteur %';
	</code>
	<br/><br/>
	<u>Exemple de requêtes pour récupérer des données</u> :
	<br/><br/>
	<code>
		# Liste des questions 
		SELECT * FROM question;
		
		# Liste des questions dont la "reponse" est "paris"
		SELECT * FROM question WHERE reponse = 'paris';
	</code>
</div>

<h2>L'interconnexion à une base de données en PHP</h2> 

<div class="phrase">
	Ci-après, on fournit un code complet permettant d'interagir avec 
	la base de données que nous avons créée. On constatera qu'une 
	liste non négligeable de fonctions est nécessaire.
	<br/><br/>
	Quel que soit ce que nous voulons faire avec la base de données, 
	il nous faudra avant toute chose nous connecter à celle-ci et 
	sélectionner la base de données que nous souhaitons manipuler :
</div>

<div class="code">
	<code>
		// Récupération d'une connexion à MySQL (le SGBD utilisé)
		$connexion = mysqli_connect("localhost", "root", "");
		// Affichage d'un message selon si la connexion a la base 
		// de données a réussi ou échoué
		if($connexion){
			// Pour éviter les problèmes d'encodage 
			mysqli_set_charset($connexion,"utf8");
			// Sélection de la base de données nommée "exercices"
			mysqli_select_db($connexion, "exercices");
			// Connexion réussie
			echo "La connexion à la base de données a été récupérée.";
		}else{
			// Connexion échouée
			echo "La connexion à la base de données est impossible.";
		}
	</code>
</div>

<div class="resultat">
	<?php
		error_reporting(E_ERROR|E_PARSE);
		// Récupération d'une connexion à MySQL (le SGBD utilisé)
		$connexion = mysqli_connect("localhost", "root", "");
		// Affichage d'un message selon si la connexion a la base 
		// de données a réussi ou échoué
		if($connexion){
			// Pour éviter les problèmes d'encodage 
			mysqli_set_charset($connexion,"utf8");
			// Sélection de la base de données nommée "exercices"
			mysqli_select_db($connexion, "exercices");
			// Connexion réussie
			echo "La connexion à la base de données a été récupérée.";
		}else{
			// Connexion échouée
			echo "La connexion à la base de données est impossible.";
		}
	?>
</div>
	
<div class="phrase">
	Occupons-nous dans un premier temps de pouvoir ajouter des 
	questions au moyen d'un formulaire :
</div>

<div class="code">
	<code>
		&lt;form method="POST" action="php/exemples/mysql.php">
			&lt;label>
				Question : &lt;br/>
				&lt;input name="question" type="text" />
			&lt;/label>
			&lt;label>
				Réponse : &lt;br/>
				&lt;input name="reponse" type="text" />
			&lt;/label>
			&lt;button>Enregistrer&lt;/button>
		&lt;/form>
		&lt;br/>
		&lt;?php 
			// Si la connexion à la base de données a été établie 
			// et si l'utilisateur a transmis sa question/réponse
			if($connexion && isset($_POST["question"], $_POST["reponse"])){
				// Pour éviter divers problèmes...
				$_POST["question"] = mysqli_escape_string($_POST["question"]);
				$_POST["reponse"] = mysqli_escape_string($_POST["reponse"]);
				// Requête SQL d'insertion à exécuter
				$sql = "
					INSERT INTO question(question, reponse) 
					VALUES('" . $_POST["question"] . "', '" . $_POST["reponse"] . "')
				";
				// On lance notre requête SQL
				$succes = mysqli_query($connexion, $sql);
				// On affiche un message selon si l'insertion a fonctionné ou non.
				if($succes){
					echo "La question a bien été enregistrée.";
				}else{
					echo "La question n'a pu être enregistrée.";
				}
			}
		?>
	</code>
</div>

<div class="resultat">
	<form method="POST" action="php/exemples/mysql.php">
		<label>
			Question : <br/>
			<input style="width: 100%;" name="question" type="text" />
		</label>
		<label>
			Réponse : <br/>
			<input style="width: 100%;" name="reponse" type="text" />
		</label>
		<button>Enregistrer</button>
	</form>
	<br/>
	<?php 
		// Si la connexion à la base de données a été établie 
		// et si l'utilisateur a transmis sa question/réponse
		if($connexion && isset($_POST["question"], $_POST["reponse"])){
			// Pour éviter divers problèmes...
			$_POST["question"] = mysqli_escape_string($connexion, $_POST["question"]);
			$_POST["reponse"] = mysqli_escape_string($connexion, $_POST["reponse"]);
			// Requête SQL d'insertion à exécuter
			$sql = "
				INSERT INTO question(question, reponse) 
				VALUES('" . $_POST["question"] . "', '" . $_POST["reponse"] . "')
			";
			// On lance notre requête SQL
			$succes = mysqli_query($connexion, $sql);
			// On affiche un message selon si l'insertion a fonctionné ou non.
			if($succes){
				echo "La question a bien été enregistrée.";
			}else{
				echo "La question n'a pu être enregistrée.";
			}
		}
	?>
</div>

<div class="phrase">
	Maintenant que nous pouvons ajouter des questions, nous souhaiterions 
	dans un second temps pouvoir en supprimer. Pour ce faire, lorsque 
	l'utilisateur cliquera sur un lien "supprimer", il nous enverra un 
	paramètre GET "numero" qui nous indiquera le numéro de la question à 
	supprimer :
</div>

<div class="code">
	<code>
		// Si la connexion à la base de données a été établie
		// et si l'utilisateur a envoyé le numéro d'une question 
		// à supprimer, on peut procéder à la suppression.
		if($connexion && isset($_GET["numero"])){
			// Pour éviter divers problèmes...
			$_GET["numero"] = mysqli_escape_string($connexion, $_GET["numero"]);
			// Requête SQL de suppression à exécuter
			$sql = "DELETE FROM question WHERE numero=" . $_GET["numero"];
			// On lance notre requête SQL
			$succes = mysqli_query($connexion, $sql);
			// On affiche un message selon si la suppression a fonctionné ou non.
			if($success){
				echo "La question a bien été supprimée.";
			}else{
				echo "Aucune question n'a été supprimée.";
			}
		}else{
			echo "Aucune question à supprimer.";
		}
	</code>
</div>

<div class="resultat">
	<?php
		// Si la connexion à la base de données a été établie
		// et si l'utilisateur a envoyé le numéro d'une question 
		// à supprimer, on peut procéder à la suppression.
		if($connexion && isset($_GET["numero"])){
			// Pour éviter divers problèmes...
			$_GET["numero"] = mysqli_escape_string($connexion, $_GET["numero"]);
			// Requête SQL de suppression à exécuter
			$sql = "DELETE FROM question WHERE numero=" . $_GET["numero"];
			// On lance notre requête SQL
			$succes = mysqli_query($connexion, $sql);
			// On affiche un message selon si la suppression a fonctionné ou non.
			if($success){
				echo "La question a bien été supprimée.";
			}else{
				echo "Aucune question n'a été supprimée.";
			}
		}else{
			echo "Aucune question à supprimer.";
		}
	?>
</div>

<div class="phrase">
	Finalement, il ne nous reste plus qu'à afficher la liste des questions 
	sous forme de tableau. On en profite pour ajouter un lien permettant 
	de supprimer une question.
</div>

<div class="code">
	<code>	
		// Si la connexion à la base de données a pu être établie
		if($connexion){
			
			// On récupère la liste des questions 
			$questions = mysqli_query($connexion, "SELECT * FROM question");
			// On récupère le nombre de questions
			$nb = mysqli_num_rows($questions);
			
			// Début du tableau listant les questions
			echo "
				&lt;table>
					&lt;tr>
						&lt;th>N°&lt;/th>
						&lt;th>Question&lt;/th>
						&lt;th>Réponse&lt;/th>
						&lt;th>Supprimer&lt;/th>
					&lt;/tr>
			";
			// On parcourt toutes les questions 
			for($i=0; $i&lt;$nb; $i++){
				// On récupère la question suivante 
				$question = mysqli_fetch_assoc($questions);
				// On affiche la question dans une ligne de tableau
				echo "
					&lt;tr>
						&lt;td>" . $question["numero"] . "&lt;/td>
						&lt;td>" . $question["question"] . "&lt;/td>
						&lt;td>" . $question["reponse"] . "&lt;/td>
						&lt;td>
							&lt;a href=\"php/exemples/mysql.php?numero=" . $question["numero"] . "\"> 
								supprimer
							&lt;/a>
						&lt;/td>
					&lt;/tr>
				";
				
			}
			// Fin du tableau listant les questions
			echo "
				&lt;/table>
			";
			
		}else{
			
			echo "La connexion à la base de données est impossible.";
			
		}
	</code>
</div>

<div class="resultat">
	<?php
		// Si la connexion à la base de données a pu être établie
		if($connexion){
			// On récupère la liste des questions 
			$questions = mysqli_query($connexion, "SELECT * FROM question");
			// On récupère le nombre de questions
			$nb = mysqli_num_rows($questions);
			// Début du tableau listant les questions
			echo "
				<table>
					<tr>
						<th>N°</th>
						<th>Question</th>
						<th>Réponse</th>
						<th>Supprimer</th>
					</tr>
			";
			// On parcourt toutes les questions 
			for($i=0; $i<$nb; $i++){
				// On récupère la question suivante 
				$question = mysqli_fetch_assoc($questions);
				// On affiche la question dans une ligne de tableau
				echo "
					<tr>
						<td>" . $question["numero"] . "</td>
						<td>" . $question["question"] . "</td>
						<td>" . $question["reponse"] . "</td>
						<td>
							<a href=\"php/exemples/mysql.php?numero=" . $question["numero"] . "\"> 
								supprimer
							</a>
						</td>
					</tr>
				";
				
			}
			// Fin du tableau listant les questions
			echo "
				</table>
			";
		}else{
			echo "La connexion à la base de données est impossible.";		
		}		
	?>
	
</div>

<?php
	
	// Pied de page
	include("../footer.php");
	
?>
