<h3>Version 3</h3>

<div class="phrase">
	<i>Fichier à compléter : <b>php/exercices/connexion/v3.php</b></i>
	<br/><br/>
	Cette fois encore, pas de grand changement... La version 3 est très 
	proche de la version 2. On se préoccupe cette fois-ci de  
	problématiques de sécurité et de confidentialité :
	<br/>
	- d'une part, en version 2, nous ne nous sommes pas souciés d'une 
	problématique de sécurité des plus classiques : l'injection SQL. 
	Quoique cela soit pardonnable à des développeurs en herbe, il n'est 
	pas admissible qu'une application web soit sensible à l'injection SQL. 
	Nous allons tenter de palier ce problème ;
	<br/>
	- d'autre part, en version 2, nous nous sommes permis de conserver 
	les mots de passe en clair en base de données ! Or, un mot de passe 
	est une information personnelle et confidentielle. Seul l'utilisateur 
	a à connaître son propre mot de passe.
	<br/><br/>
	Pour solutionner ces deux problématiques, nous adapterons ensemble 
	la procédure connecter_v2($email, $mdp) en une procédure 
	connecter_v3($email, $mdp) un peu plus complète.
	<br/><br/>
	Résolution du risque d'injection SQL :
	<br/>
	- il nous faut utiliser la fonction mysqli_escape_string($connexion, $contenu) 
	pour "échapper" une chaîne de caractères à risque $contenu ;
	<br/>
	- par exemple : $email = mysqli_escape_string($connexion, $contenu).
	<br/><br/>
	Résolution du problème de sécurité et de confidentialité lié au 
	stockage en clair des mots de passe : 
	<br/>
	- nous ne stockerons plus le mot de passe en base de données mais 
	son hash*, encore appelé empreinte ou empreinte numérique ;
	<br/>
	- lorsque l'utilisateur tente de se connecter, on ne compare plus 
	le mot de passe saisi à celui de l'utilisateur en base de données. 
	On calcule le hash du mot de passe fourni et on compare ce hash 
	à celui de l'utilisateur en base de données ;
	<br/>
	- pour hacher le mot de passe, on utilisera le bout de code suivant : 
	$mdp = hash("sha256", $mdp).
	<br/><br/>
	<i>
		* Le hash est le résultat d'une fonction de hachage. Une fonction 
		de hachage à normalement les particularités suivantes :
		<br/>
		- l'empreinte de tout contenu est normalement unique ;
		<br/>
		- la fonction de hachage est non inversible, c'est-à-dire qu'on 
		ne peut retrouver le contenu haché à partir de son empreinte. Dans 
		le cas présent, on ne peut retrouver le mot de passe à partir de 
		son empreinte.
	</i>
	<br/><br/>
	Avant toute chose, on utilisera le générateur ci-dessous afin de 
	remplacer en base de données le mot de passe des utilisateurs par 
	leur empreinte :
	<br/><br/>
	<form method="POST" action="php/exercices/connexion.php">
		<label>
			Mot de passe en clair : 
			<input name="mdp_clair" type="text" value="<?=(isset($_POST["mdp_clair"]) ? $_POST["mdp_clair"] : "")?>" /> 
			<button>Crypter</button>
		</label>
		<?php if(isset($_POST["mdp_clair"])){ ?>
		<label>
			<?=hash("sha256", $_POST["mdp_clair"])?>
		</label>
		<?php } ?>
	</form>
</div>

<div class="resultat">
	
	<?php
		
		/** 
		 * Procédure permettant de tenter d'authentifier un utilisateur 
		 * à partir de son adresse email et de son mot de passe.  
		 * L'authentification réussit si et seulement si le couple 
		 * email + mot de passe est correct. S'il est correct, on se 
		 * contente de placer son email dans le paramètre de session 
		 * "utilisateur".
		 **/
		function connecter_v3($email, $mdp){
			// [à compléter : identique à la v2 à ceci près qu'il faut 
			// "échapper" les paramètres $email et $mdp et comparer 
			// les hash des mots de passe en lieu et place des mots 
			// de passe en clairs]
		}
		
		/*
			
			/!\ IMPORTANT /!\

			Tout ce qui suit est presque identique en v2 et v3. Aussi, on 
			peut copier/coller le code de la v2. Cependant, on remplace 
			l'appel à connecter_v2(...) par connecter_v3(...).
			
		*/
		
	?>
	
</div>